#include <NDS/NDS.h>
#include <NDS/ARM9/rand.h>
#include <NDS/ARM9/console.h> //basic print funcionality

#define PT1_CR1    (*(vuint16*)0x040001A0)
#define PT1_DATA   (*(vuint16*)0x040001A2)
#define PT1_WaitBusy()   while (PT1_CR1 & /*BUSY*/0x80); 

s16 magne_x,magne_y;
s16 magne_x_ave,magne_y_ave;
s16 angle;

s16 tbl[47]=
{ 0,174,349,524,699,874,1051,1227,1405,1583,1763,1943,2125,2308,2493,2679,
  2867,3057,3249,3443,3639,3838,4040,4244,4452,4663,4877,5095,5317,5543,5773,
  6008,6248,6494,6745,7002,7265,7535,7812,8097,8390,8692,9004,9325,9656,10000,
  32767 };

s16 def[7]={ 23,12,6,3, 2,1,0 };

s16 int_atan(s16 y,s16 x)
{
	s16 a,d,i;
	if ((x==0)&&(y==0)) { return 0; }
	if ((x>=0)&&(y>=0)) {
		if (y<=x) {
			a = (y * 10000) / x;
			d = 0;
			for (i=0;i<7;i++) {
				if (a > tbl[ d + def[i] - 1 ] ) {
					d += def[i];
				}
			}
			return d;
		}
		else                { return 90-int_atan(x,y); }
	}
	else if ((x>=0)&&(y<0)) { return 360-int_atan(-y,x); }
	else if ((x<0)&&(y>=0)) { return 180-int_atan(y,-x); }
	else                    { return 180+int_atan(-y,-x); }
}


u8 SPI(u8 d)
{
	u8 res;
	PT1_DATA = d;
	PT1_WaitBusy();
	res=PT1_DATA;
	return res;
}

void hm55_reset()
{
	u16 t;
	PT1_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40 | 3;  // ~CS = L
	SPI(0x00);
	PT1_CR1 = /*E*/0x8000                 | /*MODE*/0x40 | 3;  // ~CS=H
	for (t=0;t<15000;t++) ;
}

void hm55_measure_start()
{
	u16 t;
	PT1_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40 | 3;  // ~CS = L
	SPI(0x8F);
	PT1_CR1 = /*E*/0x8000                 | /*MODE*/0x40 | 3;  // ~CS=H
	for (t=0;t<5000;t++) ;
}

u8 hm55_read_status(void)
{
	u8 res;
	PT1_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40 | 3;  // ~CS = L
	res = SPI(0xCF);
	res &= 0x0F;
	PT1_CR1 = /*E*/0x8000                 | /*MODE*/0x40 | 3;  // ~CS=H
	return res;
}

void hm55_read_data(void)
{
	u8 data1,data2,data3;
	PT1_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40 | 3;  // ~CS = L
	SPI(0xCF);
	data1 = SPI(0xFF);
	data2 = SPI(0xFF);
	data3 = SPI(0xFF);
	PT1_CR1 = /*E*/0x8000                 | /*MODE*/0x40 | 3;  // ~CS=H
	
	magne_x = ( (u16)data1<<3 ) + ( data2>>5 );  // 8bit + 3bit
	magne_y = ( ((u16)data2&0x1F )<<6 ) + ( data3>>2 );  // 5bit + 6bit
	if (magne_x & 0x0400) {  magne_x -= 0x0800;  }
	if (magne_y & 0x0400) {  magne_y -= 0x0800;  }
}

void hm55_measure(void)
{
	u8 c;
	hm55_reset(); // Reset
	while (1) {
		hm55_measure_start(); // Measurement start
		while (1) {
			c=hm55_read_status();
			if ( c!=0x00) { break; }
		}
		if (c!=0x03) {
			break;
		} else {
			hm55_reset(); // Reset
			continue;
		}
	}
	hm55_read_data();
}

//verticies for the cube
v16 CubeVectors[] = {
 		floatov16(-0.5),	floatov16(-0.5),	floatov16(0.5), 
		floatov16(0.5),		floatov16(-0.5),	floatov16(0.5),
		floatov16(0.5),		floatov16(-0.5),	floatov16(-0.5),
		floatov16(-0.5),	floatov16(-0.5),	floatov16(-0.5),
		floatov16(-0.5),	floatov16(0.5),		floatov16(0.5), 
		floatov16(0.5),		floatov16(0.5),		floatov16(0.5),
		floatov16(0.5),		floatov16(0.5),		floatov16(-0.5),
		floatov16(-0.5),	floatov16(0.5),		floatov16(-0.5)
};

//polys
u8 CubeFaces[] = {
	3,2,1,0,	0,1,5,4,	1,2,6,5,	2,3,7,6,	3,0,4,7,	5,6,7,4
};

//texture coordinates
u32 uv[] =
{
	TEXTURE_PACK(intot16(128), 0),	TEXTURE_PACK(intot16(128),intot16(128)),
	TEXTURE_PACK(0, intot16(128)),	TEXTURE_PACK(0,0)
};

u32 normals[] =
{
	NORMAL_PACK(0,intov10(-1),0),	NORMAL_PACK(0,intov10(1),0),
	NORMAL_PACK(intov10(1),0,0),	NORMAL_PACK(0,0,intov10(-1)),
	NORMAL_PACK(intov10(-1),0,0),	NORMAL_PACK(0,intov10(1),0)
};

//draw a cube face at the specified color
 void drawQuad(int poly)
{	
	
	u32 f1 = CubeFaces[poly * 4] ;	u32 f2 = CubeFaces[poly * 4 + 1] ;
	u32 f3 = CubeFaces[poly * 4 + 2] ;	u32 f4 = CubeFaces[poly * 4 + 3] ;

	glNormal(normals[poly]);

	glTexCoord1i(uv[0]);
	glVertex3v16(CubeVectors[f1*3], CubeVectors[f1*3 + 1], CubeVectors[f1*3 +  2] );
	
	glTexCoord1i(uv[1]);
	glVertex3v16(CubeVectors[f2*3], CubeVectors[f2*3 + 1], CubeVectors[f2*3 + 2] );
	
	glTexCoord1i(uv[2]);
	glVertex3v16(CubeVectors[f3*3], CubeVectors[f3*3 + 1], CubeVectors[f3*3 + 2] );

	glTexCoord1i(uv[3]);
	glVertex3v16(CubeVectors[f4*3], CubeVectors[f4*3 + 1], CubeVectors[f4*3 + 2] );
}



int main()
{
	int i;
	u16 key_st,prev_key;
	float rotateX = 45.0;
	float rotateY = 0.0;
	magne_x_ave= magne_y_ave=0;
	prev_key=0;
	powerON(POWER_LCD_TOP | POWER_LCD_BOTTOM | POWER_2D | POWER_2D_SUB |
			POWER_3D_CORE | POWER_MATRIX | POWER_SWAP_LCDS);
	videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE | ENABLE_3D);
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankC(VRAM_C_SUB_BG); 
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
	WAIT_CR &= ~0x0880;
	videoSetMode(MODE_0_3D);
	irqInitHandler(irqDefaultHandler);
	irqSet(IRQ_VBLANK, 0);
	glViewPort(0,0,255,191);
	glClearColor(0,0,0);
	glClearDepth(0x7FFF);
	vramSetBankA(VRAM_A_TEXTURE);
	
	consolePrintSet(0,1);
	consolePrintf("<<< HM55B TEST >>>\n");
	while(1) {
		hm55_measure();
		magne_x_ave = ( magne_x_ave + magne_x )/2;
		magne_y_ave = ( magne_y_ave + magne_y )/2;
		angle = int_atan(magne_y_ave,magne_x_ave);
		
		consolePrintSet(0,3);
		if (magne_x_ave!=0) { consolePrintf("M_x=%d  \n",magne_x_ave); }
			else 			{ consolePrintf("M_x=0  \n"); }
		if (magne_y_ave!=0) { consolePrintf("M_x=%d  \n",magne_y_ave); }
			else 			{ consolePrintf("M_x=0  \n"); }
		if (angle!=0)		{ consolePrintf("dir=%d  \n",angle); }
			else 			{ consolePrintf("dir=0  \n"); }
		
		rotateY = 360-angle;
		
		key_st = KEYS;
		if (key_st != prev_key) {
			//if(!(key_st & KEY_UP)) { rotateX += 5; }
			//if(!(key_st & KEY_DOWN)) { rotateX -= 5; }
		}
		if(!(key_st & KEY_UP)) { rotateX += 5; }
		if(!(key_st & KEY_DOWN)) { rotateX -= 5; }
		if (rotateX > 80) { rotateX= 80; }
		if (rotateX < 10) { rotateX= 10; }
		prev_key = key_st;
		
		
		glReset();
	
		gluPerspective(35, 256.0 / 192.0, 0.1, 40);
		gluLookAt(	0.0, 0.0, 1.0,	0.0, 0.0, 0.0,	 0.0, 1.0, 0.0);	
		
		glLight(0, RGB15(31,31,31) , 0,				  floatov10(-1.0),		 0);
		glLight(1, RGB15(31,0,31),   0,				  floatov10(1) - 1,			 0);
		glLight(2, RGB15(0,31,0) ,   floatov10(-1.0), 0,					 0);
		glLight(3, RGB15(0,0,31) ,   floatov10(1.0) - 1,  0,					 0);

		glPushMatrix();

		glTranslate3f32(0, 0, floatof32(-0.4));
		glRotateX(rotateX);
		glRotateY(rotateY);
		
		glMatrixMode(GL_TEXTURE);
		glIdentity();
		
		glMatrixMode(GL_MODELVIEW);

		glMaterialf(GL_AMBIENT, RGB15(8,8,8));
		glMaterialf(GL_DIFFUSE, RGB15(16,16,16));
		glMaterialf(GL_SPECULAR, BIT(15) | RGB15(8,8,8));
		glMaterialf(GL_EMISSION, RGB15(5,5,5));

		glMaterialShinnyness();

		glPolyFmt(POLY_ALPHA(31) | POLY_CULL_BACK | POLY_FORMAT_LIGHT0 | POLY_FORMAT_LIGHT1 | 
													POLY_FORMAT_LIGHT2 | POLY_FORMAT_LIGHT3 ) ;
		
		glBegin(GL_QUAD);
		for(i = 0; i < 6; i++) { drawQuad(i); }
		glEnd();
		
		glPopMatrix(1);
		glFlush();
	}

	return 0;
	
	
}//end main 

